# scan_100m.py : 
# try to find the External Key by scanning all 100 millions positions
import sys
import string
import getopt
import m209
from bib_util import *
from ic import IC
import HC_SCORE
#======================================
def usage():
    print "usage:"
    print "  -c crypto  The cryptogram"
    print "  -M method  The method (IC, UNI, BIG,TRI,...), default: IC"
    print "  -m file_method The file used with method"
    print "  -l letter  The letter (shift, default: Z"
    print "  -L lugs    The lugs (ex: 1:5:4:13:2:7), default 0:0:0:0:0:0"
    print "  -O overlaps    The overlaps (ex: 0:0:1...:1:1) default: 0:...0"
    print "  -P Pins    The pins (default: 0000...0000)"
    print "  -s letter  The start letter, by default 'A'"
    print "  -t number  The threshold, by default 0.065"
    print "  -I file    The Internal Key File"
    print "  -v         Verbose"
#======================================

#--------------------------------------------------
try:
  opts, args = getopt.getopt(sys.argv[1:],
    "hc:M:m:l:L:O:P:s:t:I:v", ["help", "crypto=","method=", "method_file=", 
    "letter=", "lugs=", "overlaps=", "pins=", "start_letter=", 
    "threshold=", "internal_key_file=", "verbose"])
except getopt.GetoptError as err:
  # print help information and exit:
  print sys.argv[0],": ", str(err) # option -a not recognized"
  usage()
  sys.exit(2)

#===============================================================
Wheels = [
   "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
   "ABCDEFGHIJKLMNOPQRSTUVXYZ",
   "ABCDEFGHIJKLMNOPQRSTUVX",
   "ABCDEFGHIJKLMNOPQRSTU",
   "ABCDEFGHIJKLMNOPQRS",
   "ABCDEFGHIJKLMNOPQ"
]
#===============================================================
msg=""
METHOD="IC"
file_method="IC"
cle_letter_shift = "Z"
LUGS=[0,0,0,0,0,0]
OVER=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
PINS="00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000"
START_LETTER=0
THRESHOLD=0.065
INT_KEY_FILE=""
DEBUG=False

for o, a in opts:
  if o == "-v":
    verbose = True
  elif o in ("-h", "--help"):
    usage()
    sys.exit()
  elif o in ("-c", "--crypto"):
    msg = litfic( a )
  elif o in ("-M", "--method"):
    METHOD = a
  elif o in ("-m", "--method_file"):
    file_method = a
  elif o in ("-l", "--letter"):
    cle_letter_shift = a
  elif o in ("-L", "--lugs"):
    LUGS=a.split(":")
    cle_lug=list( map(int,LUGS) )
  elif o in ("-O", "--overlaps="):
    tb = a.split(":")
    OVER = map(int,tb)
    OVER.append(0)
  elif o in ("-P", "--pins="):
    PINS = a
  elif o in ("-s", "--start_letter="):
    START_LETTER=string.find(Wheels[0],START_LETTER)
  elif o in ("-t", "--threshold="):
    THRESHOLD=float(a)
  elif o in ("-I", "--internal_key_file="):
    INT_KEY_FILE=a
  elif o in ("-v", "--verbose="):
    DEBUG=True
  else:
    assert False, "unhandled option"
if msg == "":
  usage()
  sys.exit(1)
HC_SCORE.score_init[METHOD]( file_method )

#----------------------------------------------
c = m209.Hagelin ( cle_letter_shift, 0 )
if INT_KEY_FILE != "":
    m209.restore(c,INT_KEY_FILE)
else:
    c.setAllPinsAllW( PINS )
    c.setLugs( LUGS )
    c.setOver( OVER )

#------------------------------------------------------------------
c.rewind()
#===============================================================
#=== on essaye toute les cles
maxval=0
maxkey="AAAAAA"
print "Crypto:", msg[0:50]
for i in range(START_LETTER,26):
 print Wheels[0][i]
 for j in range(25):
  print " ",Wheels[1][j]
  for k in range(23):
   for l in range(21):
    for m in range(19):
     for n in range(17):
      externalkey= Wheels[0][i]+Wheels[1][j]+Wheels[2][k]+\
                   Wheels[3][l]+Wheels[4][m]+Wheels[5][n]
      #print externalkey
      c.setExternalKey(externalkey)
      plain = m209.chiffre(c, msg)
      #print crypto
      HIGH_SCORE = HC_SCORE.score[METHOD]( plain )

      if HIGH_SCORE > THRESHOLD: 
        print externalkey, HIGH_SCORE, plain[0:30]
        if HIGH_SCORE > maxval:
            maxval = HIGH_SCORE
            maxkey = externalkey

print "MAXIMUM VALUE: ",externalkey, maxval

